%
%  MI_BEAM2.M
%
%       Creates 2-d beamformers and plot their response
%
%   Reads the data covariance matrix, with and without noise, from *.mat
%  created previously by program MP2.M. The aperture geommetry is read from
%  file ARRAY_M2.M; be sure to use the same file with program mp2.m in order
%  to have compatible data. Functions dib_mal2 and pattern2 use the last
%  parameters '...,aa,bb,cc)' to specify the figure number to be used in the
%  routine, the number of seconds to hold the plot under the pause command, 
%  and if the contour plot is desired respectively.
%
%  Miguel A. Lagunas			April 1992
%
%......................................................................
clear all;clf reset;
s=['File mp2.m defines the scenario to be used in this program'];disp(s)
%______________________________________________________________________
%  Reads array geommetry. Be sure mp2.m used previously the same aperture
array_m2;
%  coverts degrees to radians
pi=3.14159265;twopi=2*pi;ii=sqrt(-1);fiser=fisen*(pi/180);
souelr=souele*(pi/180);souazr=souazi*(pi/180);
% converts dBs to levels
for i=1:nsou;
        soule(i)=sqrt(10^(soulev(i)/10));
end;
%____________________________________________________________________
% Load data covariance matrix (with and without noise) from .mat files
load cov2.mat;load covn2.mat;
%____________________________________________________________________
%  steering vector for the desired source. assumed the first in file
% sou_l.m, which has been used previously by mp.m
sd=exp(-ii*(2*pi*fc/c)*d'.*cos(fiser'-souazr(1)*ones(length(fiser),1))*sin(souelr(1)));
%____________________________________________________________________
%  compute graphic's bounds and display data in the command window
s=['         Slowness/Azimuth polar scanning data'];disp(s);
ndibs=2500;ndib=sqrt(ndibs);ndib2=ndib/2;
aro=sin(souele*pi/180);bro=souazi*pi/180;
arx=aro.*cos(bro);ary=aro.*sin(bro);
s=['s/a of the desired ' num2str(arx(1)) ',' num2str(ary(1))];disp(s);
s=['Number of scanns in the grid ' num2str(ndib) 'x' num2str(ndib)];
% For full size s/l plot set isel equal to 1, for matched plot set to
% sources' location set any other value.
isel=1;
if isel==1;
    yymi=-20;yyma=20;xxma=20;xxmi=-20;s=['Full size s/a plot'];disp(s);
else;
    xxma=max(arx)+0.15;xxmi=min(arx)-0.15;disp(s);
    yymi=min(ary)-0.15;yyma=max(ary)+0.15;s=['Partial size s/a plot'];
end;
if xxma>1;
  xxma=1;
end;
if xxmi<-1;
  xxmi=-1;
end;
xx=xxmi:((xxma-xxmi)/ndib):xxma;

if yymi<-1;
  yymi=-1;
end;
if yyma>1;
  yyma=1;
end;
yy=yymi:((yyma-yymi)/ndib):yyma;
a1=sqrt(xxma^2+yyma^2);
if a1<1;
  a1=asin(a1);
else;
  a1=pi/2;
end;b1=atan2(yyma,xxma);
a2=sqrt(xxma^2+yymi^2);
if a2<1;
  a2=asin(a2);
else;
  a2=pi/2;
end;b2=atan2(yymi,xxma);
a3=sqrt(xxmi^2+yyma^2);
if a3<1;
  a3=asin(a3);
else;
  a3=pi/2;
end;b3=atan2(yyma,xxmi);
a4=sqrt(xxmi^2+yymi^2);
if a4<1;
  a4=asin(a4);
else;
  a4=pi/2;
end;b4=atan2(yymi,xxmi);
a5=[a1 a2 a3 a4]*180/pi;
b5=[b1 b2 b3 b4]*180/pi;
s=['S/A north-east ' num2str(a5(1)) ' elevation ' num2str(b5(1)) ' azimuth'];
disp(s);
s=['S/A north-west ' num2str(a5(2)) ' elevation ' num2str(b5(2)) ' azimuth'];
disp(s);
s=['S/A south-west ' num2str(a5(4)) ' elevation ' num2str(b5(4)) ' azimuth'];
disp(s);
s=['S/A north-east ' num2str(a5(3)) ' elevation ' num2str(b5(3)) ' azimuth'];
disp(s);
sx=exp(-ii*(2*pi*fc/c)*(d'.*cos(fiser'))*xx);
sy=exp(-ii*(2*pi*fc/c)*(d'.*sin(fiser'))*yy);
%clear s a5 b5 a1 a2 a3 a4 b1 b2 b3 b4 xxma xxmi yyma yymi;
%____________________________________________________________________
% B e a m f o r m e r s
%  The beamformers to be computed and plot are uset by seting to one
% the corresponding apat vector component. The versions available are
% the following:
%	apat(1)=1--Quiescent
%	apat(2)=1--Optimum as rx-1*sd
%	apat(3)=1--Eigenvectors response
%	apat(4)=1--MIO beamforming by blocking
%  
apat=[1 1 0 0];
%________________________________________________________________________
% quiescent
if apat(1)==1;
  stri=['quiescent response'];disp(stri);
  per=pattern2(sd,stri,sx,sy,xx,yy,ndib,souele,souazi,soulev,1,5,1);
end;
%________________________________________________________________________
% optimum as rx-1*sd
if apat(2)==1;
  arr=inv(cov)*sd;stri=['Optimum as rx-1*sd'];disp(stri);
  per=pattern2(arr,stri,sx,sy,xx,yy,ndib,souele,souazi,soulev,1,5,1);
end;
%_______________________________________________________________________
%  Eigenvectors response
if apat(3)==1;
  [u v]=eig(cov,'nobalance');auto=diag(abs(v));
  for i=1:length(auto);
    stri=['eigenvector # ' num2str(i) ' eigenvalue...' num2str(auto(i))];
    disp(stri);
    ab=u(:,i);
    per=pattern2(ab,stri,sx,sy,xx,yy,ndib,souele,souazi,soulev,1,5,1);
  end;
end;
%_______________________________________________________________________
%  MIO Beamforming with blocking
if apat(4)==1;
%  set parameters for plot functions
%	ifiga--figure for beamvector response
%	ifigb--figure for periodogram response
%       ipaua--seconds to pause beamvector response
%       ipaub--seconds to pause periodogram response
%	ipola--polar response of beamvector
%	ipolb--polar response of periodogram
  ifiga=1;ifigb=3;ipaua=3;ipaub=1;ipola=1;ipolb=1;
%  invert covariance matrix
  icov=inv(cov);
%  set attenuation level of interferences
  atlev=60;
%  compute quiescent and plot response
  ab(:,1)=sd;stri=['Quiescent'];tetam(1)=souele(1);fim(1)=souazi(1);sbm(:,1)=sd;
  cons=[sd];cbou=[1];auso=ab(:,1);
%%  alfam(1)=soulev(1);
  alfam(1)=10*log10(abs(ab(:,1)'*cov*ab(:,1)));
  per=pattern2(auso,stri,sx,sy,xx,yy,ndib,souele,souazi,soulev,ifiga,ipaua,ipola);
%  compute periodogram with null at the quiescent
  for ir=1:length(xx);
      %disp(length(xx)-ir);
    ssx=ii*(2*pi*fc/c)*d'.*cos(fiser')*xx(ir);ssx=exp(-ssx);
    for is=1:length(yy);
      if xx(ir)^2+yy(is)^2<1;
        ssy=ii*(2*pi*fc/c)*d'.*sin(fiser')*yy(is);ss=ssx.*exp(-ssy);
        u=[ss sd];rr=[1;0];auu=u'*u;
        if rcond(auu)>0.01;
          uu=u*inv(auu)*rr;v=uu;
          per1(ir,is)=10*log10(abs((v'*cov*v)/(v'*v)));
          per0(ir,is)=10*log10(abs(v'*cov*v));
        else;
          per1(ir,is)=0;
        end;
%  other procedures may be used here with more computational load
%NMLM        u=[ss sd];rr=[1;0];uu=u*inv(u'*icov*u)*rr;v=icov*uu;
%NMLM        per1(is)=10*log10(abs((uu'*v)/(v'*v)));
%NMLM        per0(is)=10*log10(abs(uu'*v));
      else; 
        per1(ir,is)=0; 
      end;  
    end;
  end;stri=['Periodogram after desired blocked'];
  per=dib_mal2(per1,stri,ndib,xx,yy,arx,ary,soulev,ifigb,ipaub,ipolb);
%  find max of the periodogram and introduce the corresponding DOA vector
  [au,du]=max(per1);[eu,fu]=max(au);sa=yy(fu);sba=du(fu);sb=xx(sba);
  tetam(2)=asin(sqrt(sa^2+sb^2))*180/pi;
  fim(2)=atan2(sa,sb)*180/pi;
  arxe(2)=sin(tetam(2)*pi/180)*cos(fim(2)*pi/180);arxe(1)=arx(1);
  arye(2)=sin(tetam(2)*pi/180)*sin(fim(2)*pi/180);arye(1)=ary(1);
  alfam(2)=per0(sba,fu);
  aux=ii*(2*pi*fc/c)*d'.*(cos(fiser')*sb+sin(fiser')*sa);  
  sbm(:,2)=exp(-aux);
%  index of the number of directions blocked plus the desired
  imm=2;
%  index to control the loop of while
  icont=1;
%  set level, below the derided level, of the resulting periodogram
%  to stop the loop
  aper=5;
%  set level of blocking
    eps=10^((alfam(1)-alfam(imm)-atlev)/10);
%  loop
  while icont==1;
%  compute beamformer with the new direction blocked, 0dB at the desired
%  and minimum norm
    cons=[cons sbm(:,imm)];cbou=[cbou eps];
    ab(:,imm)=cons*inv(cons'*cons)*cbou';
    stri=[' Beam response. Blocking..' num2str(imm-1) ' directions'];
    auso=ab(:,imm);
    alfam(1)=10*log10(abs(auso'*cov*auso));
    per=pattern2(auso,stri,sx,sy,xx,yy,ndib,souele,souazi,soulev,ifiga,ipaua,ipola);
%  compute periodogram with blocking all the directions found
    for is=1:length(xx);
        %disp(length(xx)-is);
      ssx=ii*(2*pi*fc/c)*d'.*cos(fiser')*xx(is);ssx=exp(-ssx);
      for ir=1:length(yy);ialt=0;
        if xx(is)^2+yy(ir)^2>1;
           ialt=1;
        end;
        if ialt==0;
            ssy=ii*(2*pi*fc/c)*d'.*sin(fiser')*yy(ir);ss=ssx.*exp(-ssy);
            u=[ss];rr=[1];
            for iaux=1:imm;
              u=[u sbm(:,iaux)];rr=[rr;0];
            end;auu=u'*u;
            if rcond(auu)>0.01;
              uu=u*inv(auu)*rr;v=uu;
              per1(is,ir)=10*log10(abs((v'*cov*v)/(v'*v)));
              per0(is,ir)=10*log10(abs(v'*cov*v));
            else;
              per1(is,ir)=0;
            end;
%  other procedures may be used here with more computational load
%NMLM          u=[ss sd];rr=[1;0];uu=u*inv(u'*icov*u)*rr;v=icov*uu;
%NMLM          per1(is)=10*log10(abs((uu'*v)/(v'*v)));
%NMLM          per0(is)=10*log10(abs(uu'*v));
        end; 
      end;
    end;
    stri=[' Periodogram response. Blocking..' num2str(imm-1) ' directions'];
    per=dib_mal2(per1,stri,ndib,xx,yy,arx,ary,soulev,ifigb,ipaub,ipolb);
%  Find maxima of the resulting periodogram
    [au,du]=max(per1);[eu,fu]=max(au);sa=yy(fu);sba=du(fu);sb=xx(sba);
%  Check if there are not more degrees of freedom. If yes stop the loop
    if imm<nsen;
      imm=imm+1;s=['Remain ' num2str(nsen-imm) ' degrees of freedom'];
      disp(s);
    else
      icont=0;
    end;
%  Check if the maximum of the resulting periodogram hold constraint
    if eu<alfam(1)-aper;
      icont=0;
    end;
%  Otherwise continue in finding new directions to be blocked
    tetam(imm)=asin(sqrt(sa^2+sb^2))*180/pi;
    fim(imm)=atan2(sa,sb)*180/pi;
    arxe(imm)=sin(tetam(imm)*pi/180)*cos(fim(imm)*pi/180);
    arye(imm)=sin(tetam(imm)*pi/180)*sin(fim(imm)*pi/180);
    alfam(imm)=per0(sba,fu);
    aux=ii*(2*pi*fc/c)*d'.*(cos(fiser')*sb+sin(fiser')*sa);  
    sbm(:,imm)=exp(-aux);
    eps=10^((alfam(1)-alfam(imm)-atlev)/10);
  end;
  %  Display in the command window the desired level estimated from the
  %  resulting beamformer and compare with the actual level
  iaux=length(ab(nsen,:));
  aux=abs(ab(:,iaux)'*cov*ab(:,iaux));aux=10*log10(aux);
  stri=['Desired level'];disp(stri);
  stri=['Estimate...' num2str(aux) ' dB.'];disp(stri);
  stri=['Actual.....' num2str(soulev(1)) ' dB.'];disp(stri);
end;
%____________________________________________________________________