%
%  MI_BEAM2L.M
%
%       Creates 1-d beamformers for linear arrays and plot their response
%
%   Reads the data covariance matrix, with and without noise, from *.mat
%  created previously by program MP2L.M. The aperture geommetry is read from
%  file ARRAY_M2L.M; be sure to use the same file with program mp2l.m in order
%  to have compatible data. Functions dib_mal and pattern use the last
%  parameter '...,cc)' to specify the figure number to be used in the
%  routine.
%
%  Miguel A. Lagunas			April 1992
%
%......................................................................
clear all;clf reset;
s=['File mp2l.m defines the scenario to be used in this program'];disp(s)
%______________________________________________________________________
%  Reads array geommetry. Be sure mp2.m used previously the same aperture
array_m2l;
%  coverts degrees to radians
pi=3.14159265;twopi=2*pi;ii=sqrt(-1);
souelr=souele*(pi/180);
% converts dBs to levels
for i=1:nsou;
        soule(i)=sqrt(10^(soulev(i)/10));
end;
%____________________________________________________________________
% Load data covariance matrix (with and without noise) from .mat files
load cov2l.mat;load covn2l.mat;
%____________________________________________________________________
%  steering vector for the desired source. assumed the first in file
% sou_l.m, which has been used previously by mp.m
sd=exp(-ii*(2*pi*fc/c)*d'*sin(souelr(1)));
%____________________________________________________________________
%  compute graphic's bounds and display data in the command window
s=['         Elevation scanning data'];disp(s);
ndibs=250;ndib=sqrt(ndibs);
s=['Elevation of the desired ' num2str(souele(1)) ''];disp(s);
s=['Number of scanns in the grid ' num2str(ndibs) 'x' num2str(ndib)];
%
xxe=-90:180/ndibs:90;xx=xxe*pi/180;xx=sin(xx);sx=exp(-ii*(2*pi*fc/c)*d'*xx);
%____________________________________________________________________
% B e a m f o r m e r s
%  The beamformers to be computed and plot are uset by seting to one
% the corresponding apat vector component. The versions available are
% the following:
%	apat(1)=1--Quiescent
%	apat(2)=1--Optimum as rx-1*sd
%	apat(3)=1--Eigenvectors response
%	apat(4)=1--MIO beamforming by blocking
%  
apat=[1 1 0 0];
%________________________________________________________________________
% quiescent
if apat(1)==1;
  stri=['quiescent response'];disp(stri);arr=sd;
  per=pattern(arr,stri,sx,xxe,souele,soulev,1);
end;
%________________________________________________________________________
% optimum as rx-1*sd
if apat(2)==1;
  arr=inv(cov)*sd;stri=['Optimum as rx-1*sd'];disp(stri);
  per=pattern(arr,stri,sx,xxe,souele,soulev,2);
end;
%_______________________________________________________________________
%  Eigenvectors response
if apat(3)==1;
  [u v]=eig(cov,'nobalance');auto=diag(abs(v));
  for i=1:length(auto);
    stri=['eigenvector # ' num2str(i) ' eigenvalue...' num2str(auto(i))];
    disp(stri);
    ab=u(:,i);
    per=pattern(ab,stri,sx,xxe,souele,soulev,3);pause;
  end;
end;
%_______________________________________________________________________
%  MIO Beamforming with blocking
if apat(4)==1;
%  invert covariance matrix
  icov=inv(cov);
%  set attenuation level of interferences
  atlev=40;
%  compute quiescent and plot response
  ab(:,1)=sd;stri=['Quiescent'];tetam(1)=souele(1);sbm(:,1)=sd;
  cons=[sd];cbou=[1];auso=ab(:,1);
  alfam(1)=10*log10(abs(ab(:,1)'*cov*ab(:,1)));
  per=pattern(auso,stri,sx,xxe,souele,soulev,4);
%  compute periodogram with null at the quiescent
  for ir=1:length(xxe);
    ssx=ii*(2*pi*fc/c)*d'*sin(xxe(ir)*pi/180);ssx=exp(-ssx);
    ss=ssx;u=[ss sd];rr=[1;0];auu=u'*u;
    if rcond(auu)>0.01;
       uu=u*inv(auu)*rr;v=uu;
       per1(ir)=10*log10(abs((v'*cov*v)/(v'*v)));
       per0(ir)=10*log10(abs(v'*cov*v));
    else;
       per1(ir)=0;
    end; 
  end;stri=['Periodogram after desired blocked'];
  per=dib_mal(per1,stri,xxe,souele,soulev,5);
%  find max of the periodogram and introduce the corresponding DOA vector
  [au,du]=max(per1);
  tetam(2)=xxe(du)*180/pi;
  alfam(2)=per0(du);
  aux=ii*(2*pi*fc/c)*d'*sin(tetam(2));  
  sbm(:,2)=exp(-aux);
%  index of the number of directions blocked plus the desired
  imm=2;
%  index to control the loop of while
  icont=1;
%  set level, below the derided level, of the resulting periodogram
%  to stop the loop
  aper=5;
%  set level of blocking
    eps=10^((alfam(1)-alfam(imm)-atlev)/10);
%  loop
  while icont==1;
%  compute beamformer with the new direction blocked, 0dB at the desired
%  and minimum norm
    cons=[cons sbm(:,imm)];cbou=[cbou eps];
    ab(:,imm)=cons*inv(cons'*cons)*cbou';
    stri=[' Beam response. Blocking..' num2str(imm-1) ' directions'];
    auso=ab(:,imm);
    alfam(1)=10*log10(abs(auso'*cov*auso));
    per=pattern(auso,stri,sx,xxe,souele,soulev,4);
%  compute periodogram with blocking all the directions found
    for is=1:length(xxe);
      ssx=ii*(2*pi*fc/c)*d'*sin(xxe(is)*pi/180);ssx=exp(-ssx);
      u=[ssx];rr=[1];
      for iaux=1:imm;
          u=[u sbm(:,iaux)];rr=[rr;0];
      end;
      auu=u'*u;
      if rcond(auu)>0.01;
         uu=u*inv(auu)*rr;v=uu;
         per1(is)=10*log10(abs((v'*cov*v)/(v'*v)));
         per0(is)=10*log10(abs(v'*cov*v));
      else;
         per1(is)=0; 
      end;
    end;
    stri=[' Periodogram response. Blocking..' num2str(imm-1) ' directions'];
    per=dib_mal(per1,stri,xxe,souele,soulev,5);
%  Find maxima of the resulting periodogram
    [au,du]=max(per1);
%  Check if there are not more degrees of freedom. If yes stop the loop
    if imm<nsen;
      imm=imm+1;s=['Remain ' num2str(nsen-imm) ' degrees of freedom'];
      disp(s);
    else
      icont=0;
    end;
%  Check if the maximum of the resulting periodogram hold constraint
    if au<alfam(1)-aper;
      icont=0;
    end;
%  Otherwise continue in finding new directions to be blocked
    tetam(imm)=xxe(du);
    alfam(imm)=per0(du);
    aux=ii*(2*pi*fc/c)*d'*sin(tetam(imm)*pi/180);  
    sbm(:,imm)=exp(-aux);
    eps=10^((alfam(1)-alfam(imm)-atlev)/10);
  end;
  %  Display in the command window the desired level estimated from the
  %  resulting beamformer and compare with the actual level
  iaux=length(ab(nsen,:));
  aux=abs(ab(:,iaux)'*cov*ab(:,iaux));aux=10*log10(aux);
  stri=['Desired level'];disp(stri);
  stri=['Estimate...' num2str(aux) ' dB.'];disp(stri);
  stri=['Actual.....' num2str(soulev(1)) ' dB.'];disp(stri);
end;
%____________________________________________________________________