%
%  	File MP2.M
%
%       Read the geommetry of a linear aperture from array_m2.m. From
%  this data, generates snapshots and computes data covariance matrix,
%  with and without noise; thus, writes down them in .mat files 
%
%  Miguel A. Lagunas			March 1996
%
%.........................................................................
clear all;
%_________________________________________________________________________
% Read array and source geommetry plus the number of snapshots
array_m2;
%_________________________________________________________________________
% Converts degrees to radians
pi=3.14159265;twopi=2*pi;ii=sqrt(-1);fiser=fisen*(pi/180);
souelr=souele*(pi/180);souazr=souazi*(pi/180);
% Converts dB to levels
for i=1:nsou;
        soule(i)=sqrt(10^(soulev(i)/10));pot(i)=soule(i)*soule(i);
end;
%_________________________________________________________________________
% Doa matrix of the sources
for i=1:nsou;
        dsou(:,i)=(2*pi*fc/c)*d'.*cos(fiser'-souazr(i)*ones(nsen,1));
        dsou(:,i)=dsou(:,i)*sin(souelr(i));
end;
dsoun=dsou(:,2:nsou);dsou=exp(-ii*dsou);dsoun=exp(-ii*dsoun);
luso=size(fiser,2);
ecov=dsou*diag(pot,0)*dsou'+eye(luso);
% Source waveforms
aux=diag(soule);auxn=diag(soule(2:nsou));
souwav=aux*exp(ii*(fresou'*(1:1:nsnap)));
souwavn=auxn*exp(ii*(fresou(2:nsou)'*(1:1:nsnap)));
clear aux;
%_____________________________________________________________
% Generates snapshots and computes covariance
sd=dsou(:,1);cov=zeros(nsen,nsen);covn=cov;snaa=[];
% Activate when the snapshots matrix is required to be in file
% snap.mat for further processing.
isna=0;
if isna==0;
    s=['The snapshots are not saved, just the covariance'];disp(s);
else;
    s=['Snapshots matrix saved in file snap.mat as matrix snaa'];disp(s);
end;
for i=1:nsnap;
        xsna=dsou*souwav(:,i)+randn(nsen,1)-ii*randn(nsen,1);
        cov=cov+xsna*xsna';
        snaa=[snaa xsna];
        xsna=dsoun*souwavn(:,i)+randn(nsen,1)-ii*randn(nsen,1);
        covn=covn+xsna*xsna';
end;
cov=cov/nsnap;covn=covn/nsnap;
clear souwav xsna;
%______________________________________________________________
%  Writes files .mat
% cov2.mat contains the full covariance estimate in cov and the exact
% covariance in ecov
save cov2.mat cov ecov;
s=['Cov estimate in matrix cov and exact covariance in ecov saved in file cov2.mat'];
disp(s);
% covn2.mat contains the estimate covariance without the first source
save covn2.mat covn;
s=['Cov estimate without the first source contribution saved as covn in file covn2.mat'];
disp(s);
% snap.mat contains the set of snapshots generated. Activate this option
% only when required
%save snap.mat snaa;
%___________________________________________________________________